----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    18:28:42 01/07/2008 
-- Design Name: 
-- Module Name:    cmd_decoder - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity cmd_decoder is
  port (
    clk_i : in std_logic;
    rst_i : in std_logic;
    -- Input
    switch_i : in std_logic_vector(3 downto 0);
    rxdata_i : in std_logic_vector(7 downto 0);
    rxrdy_i : in std_logic;
    -- Output
    cmd_o : out std_logic; -- 0: Clear Screen, 1: Draw Line
    point1_x_o : out std_logic_vector(6 downto 0);
    point1_y_o : out std_logic_vector(6 downto 0);
    point2_x_o : out std_logic_vector(6 downto 0);
    point2_y_o : out std_logic_vector(6 downto 0);
    rgb_o : out std_logic_vector(2 downto 0)
  );
end cmd_decoder;

architecture Behavioral of cmd_decoder is

  signal cmd : std_logic;
  signal point1_x : std_logic_vector(6 downto 0);
  signal point1_y : std_logic_vector(6 downto 0);
  signal point2_x : std_logic_vector(6 downto 0);
  signal point2_y : std_logic_vector(6 downto 0);
  signal rgb : std_logic_vector(2 downto 0);

begin

  cmd_o <= cmd;
  point1_x_o <= point1_x;
  point1_y_o <= point1_y;
  point2_x_o <= point2_x;
  point2_y_o <= point2_y;
  rgb_o <= rgb;

  process (clk_i)
  begin
    if (clk_i' event and clk_i='1') then
      if (rst_i='1') then
        cmd <= '0';
        point1_x <= (others => '0');
        point1_y <= (others => '0');
        point2_x <= (others => '0');
        point2_y <= (others => '0');
        rgb <= (others => '0');

      elsif (rxrdy_i='1') then
        case (switch_i) is
          when "0001" =>
            cmd <= rxdata_i(0);

          when "0010" =>
            point1_x <= rxdata_i(6 downto 0);

          when "0011" =>
            point1_y <= rxdata_i(6 downto 0);

          when "0100" =>
            point2_x <= rxdata_i(6 downto 0);

          when "0101" =>
            point2_y <= rxdata_i(6 downto 0);

          when "1000" =>
            rgb <= rxdata_i(2 downto 0);

          when others =>
        end case;
      end if;
    end if;
  end process;
end Behavioral;
