----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    17:22:11 01/07/2008 
-- Design Name: 
-- Module Name:    gc_top - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity gc_top is
  generic (
    ps2_use : std_logic := '1'; -- Use PS/2 instead of RS232 input?
    baudrate : std_logic_vector(1 downto 0) := "00"  -- RS232 speed: 115200 b/s
  );
  port (
    clk_i : in STD_LOGIC;
    rst_i : in STD_LOGIC;
    kbdclk_i : in STD_LOGIC;
    kbddata_i : in STD_LOGIC;
    rs232_rxd_i : in STD_LOGIC;
    draw_i : in STD_LOGIC;
    switch_i : in STD_LOGIC_VECTOR(3 downto 0);
    hsync_o : out  STD_LOGIC;
    vsync_o : out  STD_LOGIC;
    red_o : out  STD_LOGIC;
    green_o : out  STD_LOGIC;
    blue_o : out  STD_LOGIC;
    led_o : out STD_LOGIC_VECTOR(7 DOWNTO 0)
  );
end gc_top;

architecture Behavioral of gc_top is
  COMPONENT ps2controller
    PORT (
      clk_i : IN std_logic;
      rst_i : IN std_logic;
      kbdclk_i : IN std_logic;
      kbddata_i : IN std_logic;
      wefifo_o : OUT std_logic;
      data_o : OUT std_logic_vector(7 downto 0)
    );
  END COMPONENT;

  COMPONENT uart_top
  PORT (
    clk_i : IN std_logic;
    rst_i : IN std_logic;
    RxD_i : IN std_logic;
    txstart_i : IN std_logic;
    txreg_we_i : IN std_logic;
    baud_i : IN std_logic_vector(1 downto 0);
    txdata_i : IN std_logic_vector(7 downto 0);          
    rxrdy_o : OUT std_logic;
    txbusy_o : OUT std_logic;
    TxD_o : OUT std_logic;
    rxdata_o : OUT std_logic_vector(7 downto 0)
		);
  END COMPONENT;

  COMPONENT gpu_top
  PORT (
    clk_i : IN std_logic;
    rst_i : IN std_logic;
    cmd_i : IN std_logic_vector(7 downto 0);
    cmdrdy_i : IN std_logic;          
    address_o : OUT std_logic_vector(13 downto 0);
    rgb_o : OUT std_logic_vector(2 downto 0);
    rgbrdy_o : OUT std_logic;
    draw_i : in std_logic;
    switch_i : in std_logic_vector(3 downto 0);
    led_o : out std_logic_vector(7 downto 0)
    );
	END COMPONENT;

  COMPONENT vga_top
  PORT (
    clk_i : IN std_logic;
    rst_i : IN std_logic;
    we_i : IN std_logic;
    adress_w_i : IN std_logic_vector(13 downto 0);
    data_i : IN std_logic_vector(2 downto 0);          
    hsync_o : OUT std_logic;
    vsync_o : OUT std_logic;
    r_o : OUT std_logic;
    g_o : OUT std_logic;
    b_o : OUT std_logic
    );
	END COMPONENT;

  signal cmd : std_logic_vector(7 downto 0);
  signal cmdrdy : std_logic;
  signal ps2_cmd : std_logic_vector(7 downto 0);
  signal ps2_cmdrdy : std_logic;
  signal uart_cmd : std_logic_vector(7 downto 0);
  signal uart_cmdrdy : std_logic;
  signal address : std_logic_vector(13 downto 0);
  signal rgb : std_logic_vector(2 downto 0);
  signal rgbrdy : std_logic;

begin

  cmd <= ps2_cmd when ps2_use='1' else uart_cmd;
  cmdrdy <= ps2_cmdrdy when ps2_use='1' else uart_cmdrdy;

  -- Component instantiation
  -- PS2 controller
  Inst_ps2controller: ps2controller PORT MAP (
    clk_i => clk_i,
    rst_i => rst_i,
    kbdclk_i => kbdclk_i,
    kbddata_i => kbddata_i,
    wefifo_o => ps2_cmdrdy,
    data_o => ps2_cmd
  );
  -- UART controller
  Inst_uart_top: uart_top PORT MAP (
    clk_i => clk_i,
    rst_i => rst_i,
    RxD_i => rs232_rxd_i,
    txstart_i => '0',  -- Not used
    txreg_we_i => '0',  -- Not used
    baud_i => baudrate,
    txdata_i => (others => '0'),  -- Not used
    rxrdy_o => uart_cmdrdy,
    -- txbusy_o => ,  -- Not used
    -- TxD_o => ,  -- Not used
		rxdata_o => uart_cmd
	);
  -- GPU
  Inst_gpu_top: gpu_top PORT MAP (
    clk_i => clk_i,
    rst_i => rst_i,
    cmd_i => cmd,
    cmdrdy_i => cmdrdy,
    address_o => address,
    rgb_o => rgb,
    rgbrdy_o => rgbrdy,
    draw_i => draw_i,
    switch_i => switch_i,
    led_o => led_o
  );
  -- VGA controller
  Inst_vga_top: vga_top PORT MAP (
    clk_i => clk_i,
    rst_i => rst_i,
    we_i => rgbrdy,
    adress_w_i => address,
    data_i => rgb,
    hsync_o => hsync_o,
    vsync_o => vsync_o,
    r_o => red_o,
    g_o => green_o,
    b_o => blue_o
	);

end Behavioral;
