----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    17:47:35 01/08/2008 
-- Design Name: 
-- Module Name:    gpu_cu - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity gpu_cu is
  port (
    clk_i : in STD_LOGIC;
    rst_i : in STD_LOGIC;
    cmd_i : in std_logic;
    point1_x_i : in std_logic_vector(6 downto 0);
    point1_y_i : in std_logic_vector(6 downto 0);
    point2_x_i : in std_logic_vector(6 downto 0);
    point2_y_i : in std_logic_vector(6 downto 0);
    rgb_i : in std_logic_vector(2 downto 0);

    address_o : out std_logic_vector(13 downto 0);
    rgb_o : out std_logic_vector(2 downto 0);
    rgbrdy_o: out std_logic;
    -- Control
    draw_i : in std_logic
  );
end gpu_cu;

architecture Behavioral of gpu_cu is

  signal address : std_logic_vector(13 downto 0);
  signal rgb : std_logic_vector(2 downto 0);
  signal rgbrdy : std_logic;

  -- states  
  signal idle : std_logic; -- do nothing
  signal clear : std_logic; -- clearing the screen
  signal clear_output : std_logic; -- clearing the screen, output substate
  signal clear_step : std_logic; -- clearing the screen, step substate
  signal init : std_logic;  -- initializing
  signal init_dxy : std_logic;  -- initializing the dxy
  signal init_switch_xy : std_logic;  -- switching the coordinates, if necessary
  signal init_x_step : std_logic;  -- initializing the x_step
  signal draw : std_logic;  -- drawing the line
  signal draw_output : std_logic;  -- drawing the line, output substate
  signal draw_step : std_logic;  -- drawing the line, step substate
  signal draw_inc : std_logic;  -- drawing the line, inc substate
  
  -- data
  signal x1 : std_logic_vector(6 downto 0);
  signal y1 : std_logic_vector(6 downto 0);
  signal x2 : std_logic_vector(6 downto 0);
  signal y2 : std_logic_vector(6 downto 0);

  signal x : std_logic_vector(6 downto 0);
  signal y : std_logic_vector(6 downto 0);
  signal x_stop : std_logic_vector(6 downto 0);
		  
  signal dx_tmp : std_logic_vector(6 downto 0);
  signal dy_tmp : std_logic_vector(6 downto 0);
  signal dx : std_logic_vector(6 downto 0);
  signal d2x : std_logic_vector(7 downto 0);
  signal d2y : std_logic_vector(7 downto 0);

  signal p : std_logic_vector(9 downto 0);

  signal switch_xy : std_logic;
  signal x_increasing : std_logic;


begin

  address_o <= address;
  rgb_o <= rgb;
  rgbrdy_o <= rgbrdy;
  
  process (clk_i)
  begin
    if (clk_i' event and clk_i='1') then

      if rst_i='1' then
		  -- RESET signal

        address <= (others => '0');
        rgb <= (others => '0');
        rgbrdy <= '0';
		  
		  -- states
		  idle <= '1';
		  clear <= '0';
		  clear_output <= '0';
		  clear_step <= '0';
		  init <= '0';
		  init_dxy <= '0';
		  init_switch_xy <= '0';
		  init_x_step <= '0';
		  draw <= '0';
		  draw_output <= '0';
		  draw_step <= '0';
		  draw_inc <= '0';
		  
		  -- data
		  x1 <= (others => '0');
		  y1 <= (others => '0');
		  x2 <= (others => '0');
		  y2 <= (others => '0');
		  
		  x <= (others => '0');
		  y <= (others => '0');
		  x_stop <= (others => '0');
		  
		  dx_tmp <= (others => '0');
		  dy_tmp <= (others => '0');
        dx <= (others => '0');
        d2x <= (others => '0');
        d2y <= (others => '0');

        p <= (others => '0');

		  switch_xy <= '0';
		  x_increasing <= '0';

      elsif draw_i = '1' then
		  -- DRAW (= clear) signal
				
		  if cmd_i = '1' then
			 -- continue without clearing the screen
		    clear <= '0';
		    clear_output <= '0';
		    init <= '1';
		    init_dxy <= '1';
		  else
			 -- continue clearing the screen
			 clear <= '1';
			 clear_output <= '1';
  		    address <= (others => '0');
		    init <= '0';
		    init_dxy <= '0';
		  end if;
		  
		  idle <= '0';
		  clear_step <= '0';
		  init_switch_xy <= '0';
		  init_x_step <= '0';
		  draw <= '0';
		  draw_output <= '0';
		  draw_step <= '0';
		  draw_inc <= '0';

      elsif idle = '1' then
		  -- IDLE state
		
      elsif clear = '1' then
		  -- CLEAR state
		
		  if clear_output = '1' then
			 -- CLEAR state, OUTPUT substate

          rgb <= (others => '0'); -- prepare the clearing color
			 rgbrdy <= '1'; -- enable output
			 
			 clear_output <= '0';
			 clear_step <= '1';

		  elsif clear_step = '1' then
			 -- CLEAR state, STEP substate

			 rgbrdy <= '0'; -- disable output

		    if address = "11111111111111" then
			   -- loop end
			   clear <= '0';
				init <= '1';
				init_dxy <= '1';
				
			 else
			 
			   address <= address + 1;
			   clear_output <= '1';

			 end if;

			 clear_step <= '0';
	  
        end if;
		  
      elsif init = '1' then
        -- INIT state
		  
		  if init_dxy = '1' then
		    -- INIT state, DXY substate

		    -- calculate dx
			 if point1_x_i > point2_x_i then
			   dx_tmp <= point1_x_i - point2_x_i;
			 else
			   dx_tmp <= point2_x_i - point1_x_i;
			 end if;
			 
		    -- calculate dy
			 if point1_y_i > point2_y_i then
			   dy_tmp <= point1_y_i - point2_y_i;
			 else
			   dy_tmp <= point2_y_i - point1_y_i;
			 end if;

		    init_dxy <= '0';
			 init_switch_xy <= '1';
			 
		  elsif init_switch_xy = '1' then
			 -- INIT state, SWITCH_XY substate

  			 -- Check if DY/DX > 1 and switch X and Y coordinates if so
			 if dy_tmp > dx_tmp then
				dx <= dy_tmp;
				d2x <= dy_tmp & '0';
				d2y <= dx_tmp & '0';
				switch_xy <= '1';

  			   -- Check if X1 > X2 and swap points if so
			   if point1_x_i > point2_x_i then
			     x1 <= point2_y_i;
			     y1 <= point2_x_i;
			     x2 <= point1_y_i;
			     y2 <= point1_x_i;
			   else
			     x1 <= point1_y_i;
			     y1 <= point1_x_i;
			     x2 <= point2_y_i;
			     y2 <= point2_x_i;
			   end if;

			 else
			   dx <= dx_tmp;
				d2x <= dx_tmp & '0';
				d2y <= dy_tmp & '0';
				switch_xy <= '0';

  			   -- Check if Y1 > Y2 and swap points if so
			   if point1_y_i > point2_y_i then
			     x1 <= point2_x_i;
			     y1 <= point2_y_i;
			     x2 <= point1_x_i;
			     y2 <= point1_y_i;
			   else
			     x1 <= point1_x_i;
			     y1 <= point1_y_i;
			     x2 <= point2_x_i;
			     y2 <= point2_y_i;
			   end if;

			 end if;

			 init_switch_xy <= '0';
			 init_x_step <= '1';
				
		  elsif init_x_step = '1' then
		    -- INIT state, X_STEP substate

  			 -- Check if X1 > X2 and change X increment direction if so
			 if x1 > x2 then
			   x_increasing <= '0';
			 else
			   x_increasing <= '1';
			 end if;

			 x <= x1;
			 x_stop <= x2;
			 y <= y1;
				
          p <= (others => '0');

		    init_x_step <= '0';
			 init <= '0';
			 draw <= '1';
			 draw_output <= '1';
				
		  end if;
			 
		elsif draw = '1' then
		  -- DRAW state

		  if draw_output = '1' then
		    -- DRAW state, OUTPUT substate
		  
			 -- Switch X and Y coordinates if necessary
  			 if switch_xy = '1' then
			   address(13 downto 7) <= x;
			   address(6 downto 0) <= y;
			 else
			   address(13 downto 7) <= y;
			   address(6 downto 0) <= x;
			 end if;
				
          rgb <= rgb_i; -- prepare the drawing color
			 rgbrdy <= '1'; -- enable output
				
		    draw_output <= '0';
		    draw_step <= '1';

		  elsif draw_step = '1' then
		    -- DRAW state, STEP substate

          p <= p + d2y;

			 rgbrdy <= '0'; -- disable output

		    draw_step <= '0';
		    draw_inc <= '1';
		  
		  elsif draw_inc = '1' then
		    -- DRAW state, INC substate
				
			 if x = x_stop then
			   -- end of draw loop
			   draw <= '0';
				draw_inc <= '0';
 			   idle <= '1';

		    else

		      if signed(p) >= unsigned(dx) then
              y <= y + 1;
              p <= p - d2x;
            end if;

				if x_increasing = '1' then
				  x <= x + 1;
				else
				  x <= x - 1;
				end if;

		      draw_inc <= '0';
		      draw_output <= '1';

			 end if;
				
		  end if;
		  
      end if;

    end if;
  end process;
end Behavioral;

