----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    17:32:32 01/07/2008 
-- Design Name: 
-- Module Name:    gpu - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity gpu_top is
  port (
    clk_i : in STD_LOGIC;
    rst_i : in STD_LOGIC;
    cmd_i : in std_logic_vector(7 downto 0);
    cmdrdy_i : in std_logic;
    address_o : out std_logic_vector(13 downto 0);
    rgb_o : out std_logic_vector(2 downto 0);
    rgbrdy_o: out std_logic;
    -- Control
    draw_i : in std_logic;
    switch_i : in std_logic_vector(3 downto 0);
    led_o : out std_logic_vector(7 downto 0)
  );
end gpu_top;

architecture Behavioral of gpu_top is
  COMPONENT cmd_decoder
    PORT (
      clk_i : IN std_logic;
      rst_i : IN std_logic;
      switch_i : in std_logic_vector(3 downto 0);
      rxdata_i : IN std_logic_vector(7 downto 0);
      rxrdy_i : IN std_logic;          
      cmd_o : OUT std_logic;
      point1_x_o : OUT std_logic_vector(6 downto 0);
      point1_y_o : OUT std_logic_vector(6 downto 0);
      point2_x_o : OUT std_logic_vector(6 downto 0);
      point2_y_o : OUT std_logic_vector(6 downto 0);
      rgb_o : OUT std_logic_vector(2 downto 0)
    );
  END COMPONENT;

  COMPONENT gpu_cu
    PORT (
      clk_i : IN std_logic;
      rst_i : IN std_logic;
      cmd_i : IN std_logic;
      point1_x_i : IN std_logic_vector(6 downto 0);
      point1_y_i : IN std_logic_vector(6 downto 0);
      point2_x_i : IN std_logic_vector(6 downto 0);
      point2_y_i : IN std_logic_vector(6 downto 0);
      rgb_i : IN std_logic_vector(2 downto 0);
      draw_i : IN std_logic;          
      address_o : OUT std_logic_vector(13 downto 0);
      rgb_o : OUT std_logic_vector(2 downto 0);
      rgbrdy_o : OUT std_logic
    );
  END COMPONENT;

  signal cmd : std_logic;
  signal point1_x : std_logic_vector(6 downto 0);
  signal point1_y : std_logic_vector(6 downto 0);
  signal point2_x : std_logic_vector(6 downto 0);
  signal point2_y : std_logic_vector(6 downto 0);
  signal rgb : std_logic_vector(2 downto 0);

  signal led : std_logic_vector(7 downto 0);

begin

  led_o <= led;
  
  process (clk_i)
  begin
    if (clk_i' event and clk_i='1') then
      if rst_i='1' then
        led <= (others => '0');
      
      else
        case (switch_i) is
         when "0000" =>
           led <= cmd_i;

         when "0001" =>
           led <= "0000000"&cmd;

         when "0010" =>
           led <= '0'&point1_x;

         when "0011" =>
           led <= '0'&point1_y;

         when "0100" =>
           led <= '0'&point2_x;

         when "0101" =>
           led <= '0'&point2_y;
         
         when "1000" =>
           led <= "00000"&rgb;

         when others =>
           led <= (others => '0');
        end case;
      end if;
    end if;
  end process;

  -- Component instantiation
  -- Command decoder
  Inst_cmd_decoder: cmd_decoder PORT MAP (
    clk_i => clk_i,
    rst_i => rst_i,
    switch_i => switch_i,
    rxdata_i => cmd_i,
    rxrdy_i => cmdrdy_i,
    cmd_o => cmd,
    point1_x_o => point1_x,
    point1_y_o => point1_y,
    point2_x_o => point2_x,
    point2_y_o => point2_y,
    rgb_o => rgb
  );
  -- GPU control unit
  Inst_gpu_cu: gpu_cu PORT MAP (
    clk_i => clk_i,
    rst_i => rst_i,
    cmd_i => cmd,
    point1_x_i => point1_x,
    point1_y_i => point1_y,
    point2_x_i => point2_x,
    point2_y_i => point2_y,
    rgb_i => rgb,
    address_o => address_o,
    rgb_o => rgb_o,
    rgbrdy_o => rgbrdy_o,
    draw_i => draw_i
  );
end Behavioral;
