----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:02:57 11/02/2007 
-- Design Name: 
-- Module Name:    hsync - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity hsync is
	Port	(
		clk_i			: in  STD_LOGIC;
		rst_i			: in  STD_LOGIC;
		ce_i			: in  STD_LOGIC;
		Hsync_o		: out  STD_LOGIC;
		HDataOn_o	: out  STD_LOGIC;
		rowclk_o		: out  STD_LOGIC;
		column_o		: out  STD_LOGIC_VECTOR (9 downto 0)
	);
end hsync;

architecture Behavioral of hsync is

	COMPONENT counter
	GENERIC(
		data_width : integer := 11
	);
	PORT(
		ce_i		: IN std_logic;
		clk_i		: IN std_logic;
		rst_i		: IN std_logic;
		we_i		: IN std_logic;
		data_i	: IN std_logic_vector(data_width-1 downto 0);          
		data_o	: OUT std_logic_vector(data_width-1 downto 0)
		);
	END COMPONENT;

	COMPONENT comparator
	GENERIC(
		data_width : integer := 11
	);
	PORT(
		data1_i : IN std_logic_vector(data_width-1 downto 0);
		data2_i : IN std_logic_vector(data_width-1 downto 0);          
		equal_o : OUT std_logic
		);
	END COMPONENT;

	COMPONENT JK
	PORT(
		clk_i		: IN std_logic;
		ce_i		: IN std_logic;
		rst_i		: IN std_logic;
		J_i		: IN std_logic;
		K_i		: IN std_logic;          
		Q_o		: OUT std_logic
		);
	END COMPONENT;

	signal rst		: std_logic;  -- za povezavo med stevcem in "ali" vrati za reset signal
	signal rowclk	: std_logic;
	signal count	: std_logic_vector(10 downto 0); -- delaj z generiki (bulicu se ne da) 
	signal zeros	: std_logic_vector(10 downto 0);
	
	-- prva jk celica
	signal j1		: std_logic;
	signal k1		: std_logic;
	signal q1		: std_logic;
	-- druga jk celica
	signal j2		: std_logic;
	signal k2		: std_logic;
	signal q2		: std_logic;
	
	signal cmp4_out	: std_logic; -- pomozni izhod iz komparatorja, da nadomestimo komparator 4
	
begin

	-- IO Assignemnts:
	rowclk_o		<= rowclk;
	
	Hsync_o		<=	q1;
	HDataOn_o	<=	q2;
	
	column_o		<=	count(10 downto 1);
	
	-- Gates: 
	rst			<= rst_i or rowclk;
	zeros			<= (others => '0');
	rowclk		<= cmp4_out;
	j2				<= cmp4_out;
	
	-- Components instantiation:
	Inst_counter: counter
	GENERIC MAP(
		data_width => 11
	)
	PORT MAP(
		ce_i		=> ce_i,
		clk_i 	=> clk_i,
		rst_i 	=> rst,
		we_i 		=> '0',
		data_i 	=> zeros,
		data_o 	=> count
	);

	Inst_comparator_5: comparator -- Ts = 1600
	GENERIC MAP(
		data_width => 11
	)
	PORT MAP(
		data1_i 	=> count,
		data2_i 	=> conv_std_logic_vector(1599,11), --komparator se postavi prepozno, ker je steje tudi 0
		--data2_i 	=> conv_std_logic_vector(16,11),--testiranje
		equal_o 	=> cmp4_out
	);
	
	-- Ts = 1600
	-- Tdisplay = 1280
	--	Tbp = 32
	-- Tpw = 192
	-- Tfp = 96

-- komparatorja 4 ne potrebujemo, ker sta dva enaka komparatorja v vezju

	Inst_comparator_3: comparator -- Tdisplay + Tbp + Tpw, izhod je J1 v JK1
	GENERIC MAP(
		data_width => 11
	)
	PORT MAP(
		data1_i => count,
		data2_i => conv_std_logic_vector(1503,11),
		--data2_i => conv_std_logic_vector(15,11),--testiranje
		equal_o => j1
	);

	Inst_comparator_2: comparator -- Tdisplay, izhod je K2 v JK2
	GENERIC MAP(
		data_width => 11
	)
	PORT MAP(
		data1_i => count,
		data2_i => conv_std_logic_vector(1279,11),
		--data2_i => conv_std_logic_vector(12,11),--testiranje
		equal_o => k2
	);
	
	Inst_comparator_1: comparator -- Tdisplay + Tbp, izhod je K1 v JK1
	GENERIC MAP(
		data_width => 11
	)
	PORT MAP(
		data1_i => count,
		data2_i => conv_std_logic_vector(1311,11),
		--data2_i => conv_std_logic_vector(13,11),--testiranje
		equal_o => k1
	);
	
	Inst_JK_2: JK -- HDataOn_o
	PORT MAP(
		clk_i => clk_i,
		ce_i => '1',
		rst_i => rst_i,
		J_i => j2, -- ker smo odstranili komparator 4, da se ne podvaja, rowclk ima isti signal
		K_i => k2,
		Q_o => q2
	);
		
	Inst_JK_1: JK -- Hsync_o
	PORT MAP(
		clk_i => clk_i,
		ce_i => '1',
		rst_i => rst_i,
		J_i => j1,
		K_i => k1,
		Q_o => q1
	);

end Behavioral;