----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    14:07:46 01/08/2008 
-- Design Name: 
-- Module Name:    ps2controller - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity ps2controller is
Port (
clk_i : in STD_LOGIC;
rst_i : in STD_LOGIC;
kbdclk_i : in STD_LOGIC;
kbddata_i : in STD_LOGIC;
wefifo_o : out STD_LOGIC;
data_o : out STD_LOGIC_VECTOR(7 DOWNTO 0)
);
end ps2controller;
architecture Behavioral of ps2controller is
COMPONENT ps2pulsegen
PORT(
clk_i : IN std_logic;
rst_i : IN std_logic;
syncclk_i : IN std_logic;
pulse_o : OUT std_logic
);
END COMPONENT;
COMPONENT ps2cu
PORT(
clk_i : IN std_logic;
rst_i : IN std_logic;
ce_i : IN std_logic;
synckbddata_i : IN std_logic;
we_shr_o : OUT std_logic;
we_fifo_o : OUT std_logic
);
END COMPONENT;
COMPONENT ps2reg
PORT(
clk_i : IN std_logic;
rst_i : IN std_logic;
we_i : IN std_logic;
data_i : IN std_logic;
data_o : OUT std_logic_vector(8 downto 0)
);
END COMPONENT;
signal wefifo : STD_LOGIC;
signal weshr : STD_LOGIC;
signal kbdclk : STD_LOGIC;
signal kbddata : STD_LOGIC;
signal synckbdclk : STD_LOGIC;
signal synckbddata : STD_LOGIC;
signal pulse : STD_LOGIC;
signal parity : STD_LOGIC;
signal regdata : STD_LOGIC_VECTOR (8 DOWNTO 0);
begin
-- IO assignments:
kbddata <= kbddata_i;
kbdclk <= kbdclk_i;
data_o <= regdata(7 downto 0);
wefifo_o <= wefifo and parity ;
-- XOR ( Paruty Calculation) :
parity <= regdata(8) xor regdata(7) xor regdata(6)
xor regdata(5) xor regdata(4) xor regdata(3)
xor regdata(2) xor regdata(1) xor regdata(0);
-- D cell for kbdclk synhronization:
FFD1 : process ( clk_i )
begin
if (clk_i ' event and clk_i = '1') then
if rst_i = '1' then
synckbdclk <= '1';
else
synckbdclk <= kbdclk ;
end if;
end if;
end process;
-- D cell for kbddata synhronization:
FFD2 : process ( clk_i )
begin
if (clk_i ' event and clk_i = '1') then
if rst_i = '1' then
synckbddata <= '1';
else
synckbddata <= kbddata;
end if;
end if;
end process;
-- instantiate components:
-- PULSE GENERATOR:
Inst_ps2pulsegen: ps2pulsegen PORT MAP(
clk_i => clk_i ,
rst_i => rst_i ,
syncclk_i => synckbdclk ,
pulse_o => pulse
);
-- CONTROL UNIT
Inst_ps2cu: ps2cu PORT MAP(
clk_i => clk_i ,
rst_i => rst_i ,
ce_i => pulse ,
synckbddata_i => synckbddata ,
we_shr_o => weshr ,
we_fifo_o => wefifo
);
-- SHIFT REGISTER:
Inst_ps2reg: ps2reg PORT MAP(
clk_i => clk_i ,
rst_i => rst_i ,
we_i => weshr ,
data_i => kbddata ,
data_o => regdata
);
end Behavioral;
