----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    14:06:40 01/08/2008 
-- Design Name: 
-- Module Name:    ps2cu - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity ps2cu is
Port (
clk_i : in STD_LOGIC;
rst_i : in STD_LOGIC;
ce_i : in STD_LOGIC;
synckbddata_i : in STD_LOGIC;
we_shr_o : out STD_LOGIC;
we_fifo_o : out STD_LOGIC
);
end ps2cu ;
architecture Behavioral of ps2cu is
--Use descriptive names for the states :
type state_type is ( st0_IDLE , st1_START ,
st2_S0 , st3_S1 , st4_S2 , st5_S3 ,
st6_S4 , st7_S5 , st8_S6 , st9_S7 ,
st10_PARITY);
signal state , next_state : state_type;
-- Declare internal signals for all outputs of the state machine:
signal we_shr : std_logic;
signal we_fifo : std_logic;
begin
SYNC_PROC: process ( clk_i )
begin
if (clk_i ' event and clk_i = '1') then
if ( rst_i = '1') then
state <= st0_IDLE;
else
state <= next_state;
we_shr_o <= we_shr ;
we_fifo_o <= we_fifo;
end if;
end if;
end process;
--MEALY State Machine - Outputs based on state and inputs
OUTPUT_DECODE: process ( state , ce_i , synckbddata_i)
begin
-- defaults :
we_shr <= '0';
we_fifo <= '0';
--decode internal output signals:
case ( state ) is
when st0_IDLE =>
we_shr <= '0';
we_fifo <= '0';
when st1_START =>
if ce_i = '1' then
we_shr <= '1';
end if;
when st2_S0 =>
if ce_i = '1' then
we_shr <= '1';
end if;
when st3_S1 =>
if ce_i = '1' then
we_shr <= '1';
end if;
when st4_S2 =>
if ce_i = '1' then
we_shr <= '1';
end if;
when st5_S3 =>
if ce_i = '1' then
we_shr <= '1';
end if;
when st6_S4 =>
if ce_i = '1' then
we_shr <= '1';
end if;
when st7_S5 =>
if ce_i = '1' then
we_shr <= '1';
end if;
when st8_S6 =>
if ce_i = '1' then
we_shr <= '1';
end if;
when st9_S7 =>
if ce_i = '1' then
we_shr <= '1';
end if;
when st10_PARITY =>
-- received STOP bit ? :
if synckbddata_i = '1' and ce_i = '1' then
we_shr <= '0';
we_fifo <= '1';
end if;
when others =>
we_shr <= '0';
we_fifo <= '0';
end case;
end process;
NEXT_STATE_DECODE: process ( state , ce_i , synckbddata_i)
begin
--declare default state for next_state to avoid latches
next_state <= state ; --default is to stay in current state
--decode next_state:
-- change state only if pulse is present:
case ( state ) is
when st0_IDLE =>
-- received START bit ?
if ce_i = '1' and synckbddata_i = '0' then
next_state <= st1_START;
end if;
when st1_START =>
if ce_i = '1' then
next_state <= st2_S0 ;
end if;
when st2_S0 =>
if ce_i = '1' then
next_state <= st3_S1 ;
end if;
when st3_S1 =>
if ce_i = '1' then
next_state <= st4_S2 ;
end if;
when st4_S2 =>
if ce_i = '1' then
next_state <= st5_S3 ;
end if;
when st5_S3 =>
if ce_i = '1' then
next_state <= st6_S4 ;
end if;
when st6_S4 =>
if ce_i = '1' then
next_state <= st7_S5 ;
end if;
when st7_S5 =>
if ce_i = '1' then
next_state <= st8_S6 ;
end if;
when st8_S6 =>
if ce_i = '1' then
next_state <= st9_S7 ;
end if;
when st9_S7 =>
if ce_i = '1' then
next_state <= st10_PARITY;
end if;
when st10_PARITY =>
if ce_i = '1' then
next_state <= st0_IDLE;
end if;
when others =>
if ce_i = '1' then
next_state <= st0_IDLE;
end if;
end case;
end process;
end Behavioral;